<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model;

/**
 * Class Mapp
 * @package FiloBlu\MappConnector\Model
 */
class Mapp
{

    /**
     * Config Model
     *
     * @var \FiloBlu\MappConnector\Model\Config $config
     */
    public $_config;

    /**
     * Request object
     *
     * @var object
     */
    protected $_request;

    /**
     * Resource Type
     *
     * @var
     */
    protected $_resource;

    /**
     * Model loaded
     *
     * @var
     */
    protected $_model;

    /**
     * Job Record
     *
     * @var
     */
    protected $_job;

    /**
     * Operation type
     *
     * @var string
     */
    protected $_type = 'createOrUpdate';

    /**
     * @var
     */
    protected $_fileLogger;

    /**
     * Mapp constructor.
     * @param Config $config
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Config $config,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $fileLogger
    ) {
        $this->_request = (object)[];
        $this->_fileLogger = $fileLogger;
        $this->_config = $config;
    }

    /**
     * Init Request Object
     */
    protected function _init()
    {
        $this->_fileLogger->debug('Init Mapp Class', [], __METHOD__ . ':' . __LINE__);
        $this->_config->setEntity($this->_job['entity']);
        $this->_prepareRequestConfig();
    }

    /**
     * Get Entity By Type
     *
     * @return mixed
     * @throws \Exception
     */
    protected function _getEntity()
    {
        $class = '\FiloBlu\MappConnector\Model\From\Entity\\' . str_replace(' ', '', ucwords(str_replace('_', ' ', $this->_job['entity'])));
        $this->_fileLogger->debug('Get Entity ' . $class, [], __METHOD__ . ':' . __LINE__);

        if (class_exists($class)) {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $entity = $objectManager->create($class);
            return $entity;
        } else {
            $this->_fileLogger->error("The Entity class specified " . $class . " doesn't exists", [], __METHOD__ . ':' . __LINE__);
            return null;
        }
    }

    /**
     * Get Mapp Resource By Name
     *
     * @param $name
     * @return mixed
     */
    protected function _getMappResource()
    {
        try {
            $class = $this->_request->config->mapp_model;
            $this->_fileLogger->debug('Get MappResource ' . $class, [], __METHOD__ . ':' . __LINE__);
            if (!empty($class) && class_exists($class)) {
                $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
                $resource = $objectManager->create($class)->getResource();
                return $resource;
            } else {
                $this->_fileLogger->error("The Mapp Resource class specified " . $class . " doesn't exists", [], __METHOD__ . ':' . __LINE__);
                return null;
            }
        } catch (\Exception $e) {
            $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }
    }

    /**
     * Prepare config in the request object
     */
    protected function _prepareRequestConfig()
    {
        $this->_fileLogger->debug('Prepare request config', [], __METHOD__ . ':' . __LINE__);
        try {
            $entity = $this->_getEntity();
            $this->_request->config = (object)[];
            $this->_request->additional = (object)[];
            $this->_request->config->map = $this->_config->getNormalizedConfigObject();
            $this->_request->config->model = $entity->getEntityModel();
            $this->_request->config->model_primary = $entity->getEntityModelPrimary();
            $this->_request->config->adapter = $this->_config->getAdapter($this->_job['mode']);
            $this->_request->config->type = $this->_type;
            $this->_request->config->mapp_model = $entity->getEntityMappResource();
            $this->_request->config->mapp_primary = $entity->getEntityMappPrimary();
            $this->_model = $entity->getObjectData($this->_request->config->model_primary, $this->_job['entity_id']);
            $this->_request->additional->default_data = $entity->getDefaultData($this->_model);
            $this->_request->additional->additional_data = $entity->getAdditionalData($this->_model);
        } catch (\Exception $e) {
            $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }
    }

    /**
     * Prepare request in the request object
     *
     * @throws \Exception
     */
    protected function _prepareRequest()
    {
        $this->_request->request = [];
        $this->_fileLogger->debug('Prepare request', [], __METHOD__ . ':' . __LINE__);

        try {
            switch ($this->_request->config->type) {
                case 'createOrUpdate':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    $this->_request->request['post'] = [];
                    foreach ($this->_model as $k => $v) {
                        if (in_array($k, array_keys($this->_request->config->map))) {
                            $this->_request->request['post'][$this->_request->config->map[$k]] = $v;
                        }
                    }
                    break;
                case 'create':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    foreach ($this->_model as $k => $v) {
                        if (in_array($k, array_keys($this->_request->config->map))) {
                            $this->_request->request['post'][$this->_request->config->map[$k]] = $v;
                        }
                    }
                    break;
                case 'update':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    foreach ($this->_model as $k => $v) {
                        if (in_array($k, array_keys($this->_request->config->map))) {
                            $this->_request->request['post'][$this->_request->config->map[$k]] = $v;
                        }
                    }
                    break;
                case 'get':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    break;
                case 'getProfile':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    break;
                case 'delete':
                    $this->_request->request['get'][$this->_request->config->mapp_primary['primaryMapp']] = $this->_model[$this->_request->config->mapp_primary['primaryDB']];
                    break;
                default:
                    $this->_fileLogger->error('No request type specified', [], __METHOD__ . ':' . __LINE__);
            }

            if (isset($this->_request->request['post'])) {
                $this->_request->request['post'] = array_merge($this->_config->getMappedArray($this->_request->config->map, $this->_request->additional->default_data), $this->_request->request['post'], $this->_config->getMappedArray($this->_request->config->map, $this->_request->additional->additional_data));
                //Not filtered additional data
                if (!in_array($this->_job['entity'], ['subscriber'])) {
                    $this->_request->request['post'] = array_merge($this->_request->request['post'], $this->_request->additional->additional_data);
                }
                unset($this->_request->additional);
            }
        } catch (\Exception $e) {
            $this->_fileLogger->error($e->getMessage(), [], __METHOD__ . ':' . __LINE__);
        }
    }

    /**
     * Prepare Object and Make the request to the resource
     *
     * @return mixed
     * @throws \Exception
     */
    public function makeRequest()
    {
        $this->_fileLogger->debug('Make Request', $this->_job, __METHOD__ . ':' . __LINE__);
        if ($this->_config->getConfigValue('mappconnector/general/enable_export')) {
            $this->_init();
            $this->_prepareRequest();
            $resource = $this->_getMappResource();
            if (!empty($resource)) {
                $this->_fileLogger->debug('Doing request', [], __METHOD__ . ':' . __LINE__);
                return $resource->doRequest($this->_request);
            } else {
                $this->_fileLogger->error('No resource loaded', [], __METHOD__ . ':' . __LINE__);
                return null;
            }
        } else {
            echo "--------------------------------------------------------------------------------\n";
            echo "-- The export is disabled in your config. Please enable it to process records --\n";
            echo "--------------------------------------------------------------------------------\n";
            die();
        }
    }

    /**
     * Set Job in the object
     *
     * @param $job
     */
    public function setJob($job)
    {
        $this->_job = $job;
    }

    /**
     * Set Type in the object
     * possible types: [createOrUpdate,create,update,get,getProfile,update]
     *
     * @param $type
     */
    public function setType($type)
    {
        $this->_type = $type;
    }

    /**
     * Set Primary in the object
     *
     * @param $primary
     * @throws \Exception
     */
    public function setPrimary($primary)
    {
        if (in_array(strtolower(array_keys($primary)), ['primaryDB','primaryMapp'])) {
            $this->_primary = $primary;
        } else {
            $this->_fileLogger->error('Wrong associative array. Keys required ["primaryDB","primaryMapp"]', [], __METHOD__ . ':' . __LINE__);
        }
    }

    /**
     * Get the request object
     *
     * @return object
     */
    public function getRequest()
    {
        return $this->_request;
    }
}
